@@ -85,7 +85,11 @@ ostree_gpg_verifier_initable_init (GInitable        *initable,
   default_keyring_dir = g_file_new_for_path (default_keyring_path);
   if (!_ostree_gpg_verifier_add_keyring_dir (self, default_keyring_dir,
                                              cancellable, error))
-    goto out;
+    {
+      g_prefix_error (error, "Reading keyring directory '%s'",
+                      gs_file_get_path_cached (default_keyring_dir));
+      goto out;
+    }
 
   ret = TRUE;
  out:
