@@ -41,6 +41,7 @@ struct OstreeRepo {
   GFile *remote_heads_dir;
   GFile *objects_dir;
   int objects_dir_fd;
+  GFile *deltas_dir;
   GFile *uncompressed_objects_dir;
   int uncompressed_objects_dir_fd;
   GFile *remote_cache_dir;
@@ -57,6 +58,7 @@ struct OstreeRepo {
 
   gboolean inited;
   gboolean in_transaction;
+  gboolean disable_fsync;
   GHashTable *loose_object_devino_hash;
   GHashTable *updated_uncompressed_dirs;
   GHashTable *object_sizes;
