@@ -20,6 +20,9 @@
 
 #include "config.h"
 
+#include <gio/gunixinputstream.h>
+#include <gio/gunixoutputstream.h>
+
 #include "ostree-sysroot-private.h"
 #include "ostree-core-private.h"
 #include "otutil.h"
@@ -27,6 +30,253 @@
 
 #define OSTREE_DEPLOYMENT_COMPLETE_ID "dd440e3e549083b63d0efc7dc15255f1"
 
+/* FIXME when we depend on new enough libgsystem, move to
+ * gs_dfd_and_name_get_all_xattrs().
+ */
+static gboolean
+dfd_and_name_get_all_xattrs (int            dfd,
+                             const char    *name,
+                             GVariant     **out_xattrs,
+                             GCancellable  *cancellable,
+                             GError       **error)
+{
+  /* A workaround for the lack of lgetxattrat(), thanks to Florian Weimer:
+   * https://mail.gnome.org/archives/ostree-list/2014-February/msg00017.html
+   */
+  gs_free char *path = g_strdup_printf ("/proc/self/fd/%d/%s", dfd, name);
+  gs_unref_object GFile *fpath = g_file_new_for_path (path);
+  return gs_file_get_all_xattrs (fpath, out_xattrs,
+                                 cancellable, error);
+}
+
+static gboolean
+copy_one_file_fsync_at (int              src_parent_dfd,
+                        int              dest_parent_dfd,
+                        struct stat     *stbuf,
+                        const char      *name,
+                        GCancellable    *cancellable,
+                        GError         **error)
+{
+  gboolean ret = FALSE;
+  gs_unref_variant GVariant *src_xattrs = NULL;
+
+  if (!dfd_and_name_get_all_xattrs (src_parent_dfd, name,
+                                    &src_xattrs,
+                                    cancellable, error))
+    goto out;
+
+  if (S_ISREG (stbuf->st_mode))
+    {
+      /* Note the objects take ownership of the fds */
+      int src_fd = -1;
+      int dest_fd = -1;
+      gs_unref_object GInputStream *in = NULL;
+      gs_unref_object GOutputStream *out = NULL;
+
+      src_fd = openat (src_parent_dfd, name, O_RDONLY | O_NOFOLLOW | O_NOCTTY | O_NOATIME | O_CLOEXEC);
+      if (src_fd == -1)
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+      in = g_unix_input_stream_new (src_fd, TRUE);
+
+      dest_fd = openat (dest_parent_dfd, name, O_WRONLY | O_CREAT | O_EXCL | O_CLOEXEC,
+                        stbuf->st_mode);
+      if (dest_fd == -1)
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+      out = g_unix_output_stream_new (dest_fd, TRUE);
+
+      if (g_output_stream_splice (out, in, 0, cancellable, error) < 0)
+        goto out;
+
+      if (fchown (dest_fd, stbuf->st_uid, stbuf->st_gid) != 0)
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+      if (fchmod (dest_fd, stbuf->st_mode) != 0)
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+
+      if (fdatasync (dest_fd) != 0)
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+
+      /* Output stream destructor takes care of close */
+    }
+  else if (S_ISLNK (stbuf->st_mode))
+    {
+      char targetbuf[PATH_MAX+1];
+      size_t len;
+
+      do
+        len = readlinkat (src_parent_dfd, name, targetbuf, sizeof (targetbuf) - 1);
+      while (G_UNLIKELY (len == -1 && errno == EINTR));
+      if (len == -1)
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+      targetbuf[len] = '\0';
+      if (symlinkat (targetbuf, dest_parent_dfd, name) != 0)
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+      if (fchownat (dest_parent_dfd, name,
+                    stbuf->st_uid, stbuf->st_gid,
+                    AT_SYMLINK_NOFOLLOW) != 0)
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+    }
+  else
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "Unable to copy non-regular/non-symlink file '%s'",
+                   name);
+      goto out;
+    }
+
+  ret = TRUE;
+ out:
+  return ret;
+}
+
+static gboolean
+copy_dir_recurse_fsync (DIR             *src_parent_dir,
+                        int              dest_parent_dfd,
+                        const char      *name,
+                        GCancellable    *cancellable,
+                        GError         **error)
+{
+  gboolean ret = FALSE;
+  struct stat src_stbuf;
+  int src_parent_dfd = dirfd (src_parent_dir);
+  int src_dfd = -1;
+  int dest_dfd = -1;
+  DIR *srcd = NULL;
+  struct dirent *dent;
+  gs_unref_variant GVariant *xattrs = NULL;
+
+  src_dfd = openat (src_parent_dfd, name, O_RDONLY | O_NONBLOCK | O_DIRECTORY | O_CLOEXEC);
+  if (src_dfd == -1)
+    {
+      ot_util_set_error_from_errno (error, errno);
+      goto out;
+    }
+
+  /* Create with mode 0700, we'll fchmod/fchown later */
+  if (mkdirat (dest_parent_dfd, name, 0700) != 0)
+    {
+      ot_util_set_error_from_errno (error, errno);
+      goto out;
+    }
+
+  dest_dfd = openat (dest_parent_dfd, name, O_RDONLY | O_NONBLOCK | O_DIRECTORY | O_CLOEXEC);
+  if (dest_dfd == -1)
+    {
+      ot_util_set_error_from_errno (error, errno);
+      goto out;
+    }
+
+  /* Clone all xattrs first, so we get the SELinux security context
+   * right.  This will allow other users access if they have ACLs, but
+   * oh well.
+   */ 
+  if (!dfd_and_name_get_all_xattrs (src_parent_dfd, name,
+                                    &xattrs,
+                                    cancellable, error))
+    goto out;
+  if (!gs_fd_set_all_xattrs (dest_dfd, xattrs,
+                             cancellable, error))
+    goto out;
+
+  srcd = fdopendir (src_dfd);
+  if (!srcd)
+    {
+      ot_util_set_error_from_errno (error, errno);
+      goto out;
+    }
+
+  while ((dent = readdir (srcd)) != NULL)
+    {
+      const char *name = dent->d_name;
+      struct stat child_stbuf;
+
+      if (strcmp (name, ".") == 0 ||
+          strcmp (name, "..") == 0)
+        continue;
+
+      if (fstatat (src_dfd, name, &child_stbuf,
+                   AT_SYMLINK_NOFOLLOW) != 0)
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+
+      if (S_ISDIR (child_stbuf.st_mode))
+        {
+          if (!copy_dir_recurse_fsync (srcd, dest_dfd, name,
+                                       cancellable, error))
+            goto out;
+        }
+      else
+        {
+          if (!copy_one_file_fsync_at (src_dfd, dest_dfd,
+                                       &child_stbuf, name,
+                                       cancellable, error))
+            goto out;
+        }
+    }
+
+  if (fstat (src_dfd, &src_stbuf) != 0)
+    {
+      ot_util_set_error_from_errno (error, errno);
+      goto out;
+    }
+  if (fchown (dest_dfd, src_stbuf.st_uid, src_stbuf.st_gid) != 0)
+    {
+      ot_util_set_error_from_errno (error, errno);
+      goto out;
+    }
+  if (fchmod (dest_dfd, src_stbuf.st_mode) != 0)
+    {
+      ot_util_set_error_from_errno (error, errno);
+      goto out;
+    }
+
+  /* And finally, fsync the fd */
+  if (fsync (dest_dfd) != 0)
+    {
+      ot_util_set_error_from_errno (error, errno);
+      goto out;
+    }
+
+  ret = TRUE;
+ out:
+  if (srcd)
+    {
+      (void) closedir (srcd);
+      /* Note the srcd owns src_dfd */
+      src_dfd = -1;
+    }
+  if (src_dfd != -1)
+    (void) close (src_dfd);
+  if (dest_dfd != -1)
+    (void) close (dest_dfd);
+  return ret;
+}
+
 /**
  * copy_modified_config_file:
  *
@@ -46,11 +296,15 @@ copy_modified_config_file (GFile              *orig_etc,
                            GError            **error)
 {
   gboolean ret = FALSE;
+  gs_unref_object GFile *src_parent = g_file_get_parent (src);
   gs_unref_object GFileInfo *src_info = NULL;
   gs_unref_object GFileInfo *parent_info = NULL;
   gs_unref_object GFile *dest = NULL;
   gs_unref_object GFile *dest_parent = NULL;
   gs_free char *relative_path = NULL;
+  DIR *src_parent_dir = NULL;
+  int src_parent_dfd = -1;
+  int dest_parent_dfd = -1;
   
   relative_path = g_file_get_relative_path (modified_etc, src);
   g_assert (relative_path);
@@ -78,7 +332,29 @@ copy_modified_config_file (GFile              *orig_etc,
 
   if (g_file_info_get_file_type (src_info) == G_FILE_TYPE_DIRECTORY)
     {
-      if (!gs_shutil_cp_a (src, dest, cancellable, error))
+      src_parent_dfd = open (gs_file_get_path_cached (src_parent),
+                             O_RDONLY | O_NOCTTY | O_NONBLOCK | O_DIRECTORY | O_CLOEXEC);
+      if (src_parent_dfd == -1)
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+      src_parent_dir = fdopendir (src_parent_dfd);
+      if (!src_parent_dir)
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+      dest_parent_dfd = open (gs_file_get_path_cached (dest_parent),
+                              O_RDONLY | O_NOCTTY | O_NONBLOCK | O_DIRECTORY | O_CLOEXEC);
+      if (dest_parent_dfd == -1)
+        {
+          ot_util_set_error_from_errno (error, errno);
+          goto out;
+        }
+      if (!copy_dir_recurse_fsync (src_parent_dir, dest_parent_dfd,
+                                   gs_file_get_basename_cached (src),
+                                   cancellable, error))
         goto out;
     }
   else
@@ -86,10 +362,26 @@ copy_modified_config_file (GFile              *orig_etc,
       if (!g_file_copy (src, dest, G_FILE_COPY_OVERWRITE | G_FILE_COPY_NOFOLLOW_SYMLINKS | G_FILE_COPY_ALL_METADATA,
                         cancellable, NULL, NULL, error))
         goto out;
+      if (g_file_info_get_file_type (src_info) == G_FILE_TYPE_REGULAR)
+        {
+          if (!gs_file_sync_data (dest, cancellable, error))
+            goto out;
+        }
+      if (!ot_util_fsync_directory (dest_parent, cancellable, error))
+        goto out;
     }
 
   ret = TRUE;
  out:
+  if (src_parent_dir)
+    {
+      (void) closedir (src_parent_dir);
+      src_parent_dfd = -1;
+    }
+  if (src_parent_dfd != -1)
+    (void) close (src_parent_dfd);
+  if (dest_parent_dfd != -1)
+    (void) close (dest_parent_dfd);
   return ret;
 }
 
@@ -220,7 +512,7 @@ checkout_deployment_tree (OstreeSysroot     *sysroot,
   deploy_target_path = g_file_get_child (osdeploy_path, checkout_target_name);
 
   deploy_parent = g_file_get_parent (deploy_target_path);
-  if (!gs_file_ensure_directory (deploy_parent, TRUE, cancellable, error))
+  if (!ot_util_ensure_directory_and_fsync (deploy_parent, cancellable, error))
     goto out;
   
   g_print ("ostadmin: Creating deployment %s\n",
@@ -569,16 +861,21 @@ write_origin_file (OstreeSysroot         *sysroot,
     {
       gs_unref_object GFile *deployment_path = ostree_sysroot_get_deployment_directory (sysroot, deployment);
       gs_unref_object GFile *origin_path = ostree_sysroot_get_deployment_origin_path (deployment_path);
+      gs_unref_object GFile *origin_parent = g_file_get_parent (origin_path);
       gs_free char *contents = NULL;
       gsize len;
+      gs_unref_bytes GBytes *contents_bytes = NULL;
 
       contents = g_key_file_to_data (origin, &len, error);
       if (!contents)
         goto out;
+      contents_bytes = g_bytes_new_static (contents, len);
 
-      if (!g_file_replace_contents (origin_path, contents, len, NULL, FALSE,
-                                    G_FILE_CREATE_REPLACE_DESTINATION, NULL,
-                                    cancellable, error))
+      if (!ot_gfile_replace_contents_fsync (origin_path, contents_bytes,
+                                            cancellable, error))
+        goto out;
+
+      if (!ot_util_fsync_directory (origin_parent, cancellable, error))
         goto out;
     }
 
@@ -732,7 +1029,7 @@ swap_bootlinks (OstreeSysroot *self,
   ostree_subbootdir_name = g_strdup_printf ("boot.%d.%d", bootversion, new_subbootversion);
   ostree_subbootdir = g_file_resolve_relative_path (ostree_dir, ostree_subbootdir_name);
 
-  if (!gs_file_ensure_directory (ostree_subbootdir, TRUE, cancellable, error))
+  if (!ot_util_ensure_directory_and_fsync (ostree_subbootdir, cancellable, error))
     goto out;
 
   for (i = 0; i < new_deployments->len; i++)
@@ -749,9 +1046,9 @@ swap_bootlinks (OstreeSysroot *self,
       gs_unref_object GFile *linkname = g_file_get_child (ostree_subbootdir, bootlink_pathname);
       gs_unref_object GFile *linkname_parent = g_file_get_parent (linkname);
 
-      if (!gs_file_ensure_directory (linkname_parent, TRUE, cancellable, error))
+      if (!ot_util_ensure_directory_and_fsync (linkname_parent, cancellable, error))
         goto out;
-
+      
       if (!g_file_make_symbolic_link (linkname, bootlink_target, cancellable, error))
         goto out;
     }
@@ -866,10 +1163,10 @@ install_deployment_kernel (OstreeSysroot   *sysroot,
                                                new_bootversion, osname, 
                                                ostree_deployment_get_index (deployment));
 
-  if (!gs_file_ensure_directory (bootcsumdir, TRUE, cancellable, error))
+  if (!ot_util_ensure_directory_and_fsync (bootcsumdir, cancellable, error))
     goto out;
   bootconfpath_parent = g_file_get_parent (bootconfpath);
-  if (!gs_file_ensure_directory (bootconfpath_parent, TRUE, cancellable, error))
+  if (!ot_util_ensure_directory_and_fsync (bootconfpath_parent, cancellable, error))
     goto out;
 
   dest_kernel_name = remove_checksum_from_kernel_name (gs_file_get_basename_cached (tree_kernel_path),
@@ -952,6 +1249,12 @@ install_deployment_kernel (OstreeSysroot   *sysroot,
                                cancellable, error))
       goto out;
 
+  /* Now sync out parent directories */
+  if (!ot_util_fsync_directory (bootcsumdir, cancellable, error))
+    goto out;
+  if (!ot_util_fsync_directory (bootconfpath_parent, cancellable, error))
+    goto out;
+
   ret = TRUE;
  out:
   return ret;
@@ -1005,41 +1308,40 @@ assign_bootserials (GPtrArray   *deployments)
   return ret;
 }
 
-static GHashTable *
-bootconfig_counts_for_deployment_list (GPtrArray   *deployments)
+static gboolean
+deployment_bootconfigs_equal (OstreeDeployment *a,
+                              OstreeDeployment *b)
 {
-  guint i;
-  GHashTable *ret = 
-    g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
+  const char *a_bootcsum = ostree_deployment_get_bootcsum (a);
+  const char *b_bootcsum = ostree_deployment_get_bootcsum (b);
 
-  for (i = 0; i < deployments->len; i++)
-    {
-      OstreeDeployment *deployment = deployments->pdata[i];
-      const char *bootcsum = ostree_deployment_get_bootcsum (deployment);
-      OstreeBootconfigParser *bootconfig = ostree_deployment_get_bootconfig (deployment);
-      const char *boot_options = ostree_bootconfig_parser_get (bootconfig, "options");
-      GChecksum *bootconfig_checksum = g_checksum_new (G_CHECKSUM_SHA256);
-      const char *bootconfig_checksum_str;
-      __attribute__((cleanup(_ostree_kernel_args_cleanup))) OstreeKernelArgs *kargs = NULL;
-      gs_free char *boot_options_without_ostree = NULL;
-      guint count;
+  if (strcmp (a_bootcsum, b_bootcsum) != 0)
+    return FALSE;
+
+  {
+    OstreeBootconfigParser *a_bootconfig = ostree_deployment_get_bootconfig (a);
+    OstreeBootconfigParser *b_bootconfig = ostree_deployment_get_bootconfig (b);
+    const char *a_boot_options = ostree_bootconfig_parser_get (a_bootconfig, "options");
+    const char *b_boot_options = ostree_bootconfig_parser_get (b_bootconfig, "options");
+    __attribute__((cleanup(_ostree_kernel_args_cleanup))) OstreeKernelArgs *a_kargs = NULL;
+    __attribute__((cleanup(_ostree_kernel_args_cleanup))) OstreeKernelArgs *b_kargs = NULL;
+    gs_free char *a_boot_options_without_ostree = NULL;
+    gs_free char *b_boot_options_without_ostree = NULL;
       
-      /* We checksum the kernel arguments *except* ostree= */
-      kargs = _ostree_kernel_args_from_string (boot_options);
-      _ostree_kernel_args_replace (kargs, "ostree");
-      boot_options_without_ostree = _ostree_kernel_args_to_string (kargs);
+    /* We checksum the kernel arguments *except* ostree= */
+    a_kargs = _ostree_kernel_args_from_string (a_boot_options);
+    _ostree_kernel_args_replace (a_kargs, "ostree");
+    a_boot_options_without_ostree = _ostree_kernel_args_to_string (a_kargs);
 
-      g_checksum_update (bootconfig_checksum, (guint8*)bootcsum, strlen (bootcsum));
-      g_checksum_update (bootconfig_checksum, (guint8*)boot_options_without_ostree,
-                         strlen (boot_options_without_ostree));
+    b_kargs = _ostree_kernel_args_from_string (b_boot_options);
+    _ostree_kernel_args_replace (b_kargs, "ostree");
+    b_boot_options_without_ostree = _ostree_kernel_args_to_string (b_kargs);
 
-      bootconfig_checksum_str = g_checksum_get_string (bootconfig_checksum);
+    if (strcmp (a_boot_options_without_ostree, b_boot_options_without_ostree) != 0)
+      return FALSE;
+  }
 
-      count = GPOINTER_TO_UINT (g_hash_table_lookup (ret, bootconfig_checksum_str));
-      g_hash_table_replace (ret, g_strdup (bootconfig_checksum_str),
-                            GUINT_TO_POINTER (count + 1));
-    }
-  return ret;
+  return TRUE;
 }
 
 /* TEMPORARY HACK: Add a "current" symbolic link that's easy to
@@ -1120,21 +1422,10 @@ ostree_sysroot_write_deployments (OstreeSysroot     *self,
     requires_new_bootversion = TRUE;
   else
     {
-      GHashTableIter hashiter;
-      gpointer hkey, hvalue;
-      gs_unref_hashtable GHashTable *new_bootconfig_to_count = 
-        bootconfig_counts_for_deployment_list (new_deployments);
-      gs_unref_hashtable GHashTable *orig_bootconfig_to_count
-        = bootconfig_counts_for_deployment_list (self->deployments);
-
-      g_hash_table_iter_init (&hashiter, orig_bootconfig_to_count);
-      while (g_hash_table_iter_next (&hashiter, &hkey, &hvalue))
+      for (i = 0; i < new_deployments->len; i++)
         {
-          guint orig_count = GPOINTER_TO_UINT (hvalue);
-          gpointer new_countp = g_hash_table_lookup (new_bootconfig_to_count, hkey);
-          guint new_count = GPOINTER_TO_UINT (new_countp);
-
-          if (orig_count != new_count)
+          if (!deployment_bootconfigs_equal (new_deployments->pdata[i],
+                                             self->deployments->pdata[i]))
             {
               requires_new_bootversion = TRUE;
               break;
@@ -1145,9 +1436,19 @@ ostree_sysroot_write_deployments (OstreeSysroot     *self,
   for (i = 0; i < new_deployments->len; i++)
     {
       OstreeDeployment *deployment = new_deployments->pdata[i];
+      gs_unref_object GFile *deployment_root = NULL;
       
       if (deployment == self->booted_deployment)
         found_booted_deployment = TRUE;
+      
+      deployment_root = ostree_sysroot_get_deployment_directory (self, deployment);
+      if (!g_file_query_exists (deployment_root, NULL))
+        {
+          g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                       "Unable to find expected deployment root: %s",
+                       gs_file_get_path_cached (deployment_root));
+          goto out;
+        }
 
       ostree_deployment_set_index (deployment, i);
     }
@@ -1182,7 +1483,7 @@ ostree_sysroot_write_deployments (OstreeSysroot     *self,
 
       new_loader_entries_dir = ot_gfile_resolve_path_printf (self->path, "boot/loader.%d/entries",
                                                              new_bootversion);
-      if (!gs_file_ensure_directory (new_loader_entries_dir, TRUE, cancellable, error))
+      if (!ot_util_ensure_directory_and_fsync (new_loader_entries_dir, cancellable, error))
         goto out;
       
       for (i = 0; i < new_deployments->len; i++)
@@ -1205,6 +1506,9 @@ ostree_sysroot_write_deployments (OstreeSysroot     *self,
           goto out;
         }
 
+      g_debug ("Using bootloader: %s", bootloader ?
+               g_type_name (G_TYPE_FROM_INSTANCE (bootloader)) : "(none)");
+
       if (bootloader && !_ostree_bootloader_write_config (bootloader, new_bootversion,
                                                           cancellable, error))
         {
