@@ -25,6 +25,7 @@
 #include <gio/gio.h>
 #include <gio/gunixinputstream.h>
 #include <gio/gunixoutputstream.h>
+#include <gio/gfiledescriptorbased.h>
 
 #include <string.h>
 
@@ -247,7 +248,7 @@ ot_gfile_load_contents_utf8_allow_noent (GFile          *path,
                                          GCancellable   *cancellable,
                                          GError        **error)
 {
-  gboolean ret = TRUE;
+  gboolean ret = FALSE;
   GError *temp_error = NULL;
   gs_free char *ret_contents = NULL;
 
@@ -271,6 +272,78 @@ ot_gfile_load_contents_utf8_allow_noent (GFile          *path,
   return ret;
 }
 
+/**
+ * ot_gfile_replace_contents_fsync:
+ * 
+ * Like g_file_replace_contents(), except always uses fdatasync().
+ */
+gboolean
+ot_gfile_replace_contents_fsync (GFile          *path,
+                                 GBytes         *contents,
+                                 GCancellable   *cancellable,
+                                 GError        **error)
+{
+  gboolean ret = FALSE;
+  int parent_dfd;
+  int fd;
+  const char *target_basename = gs_file_get_basename_cached (path);
+  gs_unref_object GFile *parent = NULL;
+  gs_free char *tmpname = NULL;
+  gs_unref_object GOutputStream *stream = NULL;
+  gs_unref_object GInputStream *instream = NULL;
+
+  parent = g_file_get_parent (path);
+
+  if (!gs_file_open_dir_fd (parent, &parent_dfd,
+                            cancellable, error))
+    goto out;
+
+  if (!gs_file_open_in_tmpdir_at (parent_dfd, 0644,
+                                  &tmpname, &stream,
+                                  cancellable, error))
+    goto out;
+
+  g_assert (G_IS_FILE_DESCRIPTOR_BASED (stream));
+  fd = g_file_descriptor_based_get_fd (G_FILE_DESCRIPTOR_BASED (stream));
+
+  instream = g_memory_input_stream_new_from_bytes (contents);
+
+  if (posix_fallocate (fd, 0, g_bytes_get_size (contents)) != 0)
+    {
+      ot_util_set_error_from_errno (error, errno);
+      goto out;
+    }
+
+  if (g_output_stream_splice (stream, instream, 0,
+                              cancellable, error) < 0)
+    goto out;
+
+  if (fdatasync (fd) != 0)
+    {
+      ot_util_set_error_from_errno (error, errno);
+      goto out;
+    }
+
+  if (!g_output_stream_close (stream, cancellable, error))
+    goto out;
+
+  if (renameat (parent_dfd, tmpname, parent_dfd, target_basename) == -1)
+    {
+      ot_util_set_error_from_errno (error, errno);
+      goto out;
+    }
+
+  g_clear_pointer (&tmpname, g_free);
+
+  ret = TRUE;
+ out:
+  if (tmpname)
+    (void) unlinkat (parent_dfd, tmpname, 0);
+  if (parent_dfd != -1)
+    (void) close (parent_dfd);
+  return ret;
+}
+
 /**
  * ot_gfile_ensure_unlinked:
  *
@@ -303,6 +376,107 @@ ot_gfile_ensure_unlinked (GFile         *path,
   return ret;
 }
 
+/**
+ * ot_util_fsync_directory:
+ * @dir: Path to a directory
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Ensure that all entries in directory @dir are on disk.
+ */
+gboolean
+ot_util_fsync_directory (GFile         *dir,
+                         GCancellable  *cancellable,
+                         GError       **error)
+{
+  gboolean ret = FALSE;
+  int dfd = -1;
+
+  if (!gs_file_open_dir_fd (dir, &dfd, cancellable, error))
+    goto out;
+
+  if (fsync (dfd) != 0)
+    {
+      ot_util_set_error_from_errno (error, errno);
+      goto out;
+    }
+
+  ret = TRUE;
+ out:
+  if (dfd != -1)
+    (void) close (dfd);
+  return ret;
+}
+
+/**
+ * ot_util_ensure_directory_and_fsync:
+ * @dir: Path to a directory
+ * @cancellable: Cancellable
+ * @error: Error
+ *
+ * Create @dir (and all intermediate parent directories), ensuring
+ * that all entries are on disk.
+ */
+gboolean
+ot_util_ensure_directory_and_fsync (GFile         *dir,
+                                    GCancellable  *cancellable,
+                                    GError       **error)
+{
+  gboolean ret = FALSE;
+  int parentfd = -1;
+  const char *basename = gs_file_get_basename_cached (dir);
+  gs_unref_object GFile *parent = g_file_get_parent (dir);
+  
+ again:
+  parentfd = open (gs_file_get_path_cached (parent),
+                   O_RDONLY | O_NONBLOCK | O_DIRECTORY | O_CLOEXEC);
+  if (parentfd == -1)
+    {
+      if (errno == ENOENT)
+        {
+          if (!ot_util_ensure_directory_and_fsync (parent, cancellable, error))
+            goto out;
+          goto again;
+        }
+      else
+        {
+          int errsv = errno;
+          g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errsv),
+                       "opendir: %s", g_strerror (errsv));
+          goto out;
+        }
+    }
+  
+  if (mkdirat (parentfd, basename, 0777) == -1)
+    {
+      if (errno == EEXIST)
+        {
+          ;
+        }
+      else
+        {
+          int errsv = errno;
+          g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errsv),
+                       "mkdirat: %s", g_strerror (errsv));
+          goto out;
+        }
+    }
+
+  if (fsync (parentfd) == -1)
+    {
+      int errsv = errno;
+      g_set_error (error, G_IO_ERROR, g_io_error_from_errno (errsv),
+                   "fsync: %s", g_strerror (errsv));
+      goto out;
+    }
+
+  ret = TRUE;
+ out:
+  if (parentfd != -1)
+    (void) close (parentfd);
+  return ret;
+}
+
 /**
  * ot_gfile_atomic_symlink_swap:
  * @path: Replace the contents of this symbolic link
@@ -325,6 +499,7 @@ ot_gfile_atomic_symlink_swap (GFile          *path,
   gs_unref_object GFile *parent = g_file_get_parent (path);
   gs_free char *tmpname = g_strconcat (gs_file_get_basename_cached (path), ".tmp", NULL);
   gs_unref_object GFile *tmppath = g_file_get_child (parent, tmpname);
+  int parent_dfd = -1;
 
   if (!ot_gfile_ensure_unlinked (tmppath, cancellable, error))
     goto out;
@@ -332,10 +507,28 @@ ot_gfile_atomic_symlink_swap (GFile          *path,
   if (!g_file_make_symbolic_link (tmppath, target, cancellable, error))
     goto out;
 
+  if (!gs_file_open_dir_fd (parent, &parent_dfd, cancellable, error))
+    goto out;
+
+  /* Ensure the link has hit disk */
+  if (fsync (parent_dfd) != 0)
+    {
+      ot_util_set_error_from_errno (error, errno);
+      goto out;
+    }
+
   if (!gs_file_rename (tmppath, path, cancellable, error))
     goto out;
 
+  /* And sync again for good measure */
+  if (fsync (parent_dfd) != 0)
+    {
+      ot_util_set_error_from_errno (error, errno);
+      goto out;
+    }
+
   ret = TRUE;
  out:
+  if (parent_dfd != -1) (void) close (parent_dfd);
   return ret;
 }
