@@ -69,6 +69,11 @@ gboolean ot_gfile_load_contents_utf8_allow_noent (GFile          *path,
                                                   GCancellable   *cancellable,
                                                   GError        **error);
 
+gboolean ot_gfile_replace_contents_fsync (GFile          *path,
+                                          GBytes         *contents,
+                                          GCancellable   *cancellable,
+                                          GError        **error);
+
 gboolean ot_gfile_ensure_unlinked (GFile         *path,
                                    GCancellable  *cancellable,
                                    GError       **error);
@@ -78,5 +83,13 @@ gboolean ot_gfile_atomic_symlink_swap (GFile          *path,
                                        GCancellable   *cancellable,
                                        GError        **error);
 
+gboolean ot_util_ensure_directory_and_fsync (GFile         *dir,
+                                             GCancellable  *cancellable,
+                                             GError       **error);
+
+gboolean ot_util_fsync_directory (GFile         *dir,
+                                  GCancellable  *cancellable,
+                                  GError       **error);
+
 G_END_DECLS
 
