@@ -174,9 +174,10 @@ ot_admin_builtin_deploy (int argc, char **argv, OstreeSysroot *sysroot, GCancell
       goto out;
   }
 
-  if (!ot_admin_complete_deploy_one (sysroot, opt_osname,
-                                     new_deployment, merge_deployment, opt_retain,
-                                     cancellable, error))
+  if (!ostree_sysroot_simple_write_deployment (sysroot, opt_osname,
+                                               new_deployment, merge_deployment,
+                                               opt_retain ? OSTREE_SYSROOT_SIMPLE_WRITE_DEPLOYMENT_FLAGS_RETAIN : 0,
+                                               cancellable, error))
     goto out;
 
   ret = TRUE;
